# Generated by Django 4.0.3 on 2022-03-23 10:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authorities', '0003_authority_email'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authority',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Name'),
        ),
        migrations.AlterUniqueTogether(
            name='authority',
            unique_together={('name', 'kind')},
        ),
    ]



from django.shortcuts import render
from django.views.generic import ListView, CreateView, UpdateView, DetailView
from .models import Authority
from .forms import AuthorityUsersModelForm


class AuthorityListView(ListView):
    model = Authority
    context_object_name = "authorities"


class AuthorityCreateView(CreateView):
    model = Authority
    fields = (
        "name",
        "kind",
        "is_active",
        "parent",
        "email",
    )


class AuthorityUpdateView(UpdateView):
    model = Authority
    context_object_name = "authority"
    fields = (
        "name",
        "kind",
        "is_active",
        "parent",
        "email",
    )


class AuthorityDetailView(DetailView):
    model = Authority
    context_object_name = "authority"


class AuthorityEditUsersView(
    UpdateView,
):
    model = Authority
    form_class = AuthorityUsersModelForm
    # template_name='authorities/authority_edit_users.html'
    context_object_name = "authority"

    def get_form_kwargs(self):
        kwargs = super(AuthorityEditUsersView, self).get_form_kwargs()
        return kwargs



from django.db import models
from django.conf import settings
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
import importlib


def default_name_str(self):
    return "{0}: {1}".format(self.id, self.name)


def get_function(function_name):
    parts = function_name.split(".")
    name = parts[-1]
    module_path = ".".join(parts[:-1])
    module = importlib.import_module(module_path)
    return getattr(module, name)


try:
    authority_str_function = get_function(settings.AUTHORITY_STR_FUNCTION)
except:
    authority_str_function = default_name_str

try:
    authority_kind_str_function = get_function(settings.AUTHORITY_KIND_STR_FUNCTION)
except:
    authority_kind_str_function = default_name_str


class NamedModel(models.Model):
    
    code = models.CharField(
        max_length=32,
        blank=True,
        null=True,
        help_text=_(
            "Optional field to add an organization-specific code in addition to the name"
        ),
    )

    class Meta:
        abstract = True


class AuthorityKind(NamedModel):
    name = models.CharField(max_length=255, verbose_name=_("Name"), unique=True)
    class Meta:
        verbose_name = _("Authority Kind")
        verbose_name_plural = _("Authority Kinds")

    def __str__(self):
        return authority_kind_str_function(self)


class Authority(NamedModel):
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    kind = models.ForeignKey(
        "AuthorityKind",
        verbose_name=_(
            "Kind",
        ),
        on_delete=models.PROTECT,
    )
    is_active = models.BooleanField(
        default=True,
        help_text=_("Unselect this if the authority is not active any more"),
    )
    parent = models.ForeignKey(
        "self",
        verbose_name=_(
            "Parent",
        ),
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )
    email = models.EmailField(
        verbose_name=_(
            "Email",
        ),
        blank=True,
        null=True,
    )
    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        verbose_name=_("Authority users"),
        related_name="authorities",
        blank=True,
    )

    def get_absolute_url(self):
        return reverse("authority_view", args=[str(self.id)])

    class Meta:
        verbose_name = _("Authority")
        verbose_name_plural = _("Authorities")
        unique_together = ("name", "kind")

    def __str__(self):
        return authority_str_function(self)



# Generated by Django 2.0.6 on 2018-06-08 08:19

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Authority",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(max_length=255, unique=True, verbose_name="Όνομα"),
                ),
                (
                    "code",
                    models.CharField(
                        blank=True,
                        help_text="Optional field to add an organization-specific code in addition to the name",
                        max_length=32,
                        null=True,
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Unselect this if the authority is not active any more",
                    ),
                ),
            ],
            options={
                "verbose_name": "Authority",
                "verbose_name_plural": "Authorities",
            },
        ),
        migrations.CreateModel(
            name="AuthorityKind",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(max_length=255, unique=True, verbose_name="Όνομα"),
                ),
                (
                    "code",
                    models.CharField(
                        blank=True,
                        help_text="Optional field to add an organization-specific code in addition to the name",
                        max_length=32,
                        null=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Authority Kind",
                "verbose_name_plural": "Authority Kinds",
            },
        ),
        migrations.AddField(
            model_name="authority",
            name="kind",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="authorities.AuthorityKind",
                verbose_name="Kind",
            ),
        ),
        migrations.AddField(
            model_name="authority",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="authorities.Authority",
                verbose_name="Parent",
            ),
        ),
        migrations.AddField(
            model_name="authority",
            name="users",
            field=models.ManyToManyField(
                blank=True,
                related_name="authorities",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Authority users",
            ),
        ),
    ]



from django.contrib import admin
from django.urls import path
from .views import (
    AuthorityListView,
    AuthorityCreateView,
    AuthorityUpdateView,
    AuthorityDetailView,
    AuthorityEditUsersView,
)
from django.contrib.auth.decorators import permission_required

urlpatterns = [
    path(
        "",
        permission_required("authorities.view_authority")(AuthorityListView.as_view()),
        name="authority_list",
    ),
    path(
        "create",
        permission_required("authorities.add_authority")(AuthorityCreateView.as_view()),
        name="authority_create",
    ),
    path(
        "update/<int:pk>/",
        permission_required("authorities.change_authority")(
            AuthorityUpdateView.as_view()
        ),
        name="authority_update",
    ),
    path(
        "view/<int:pk>/",
        permission_required("authorities.view_authority")(
            AuthorityDetailView.as_view()
        ),
        name="authority_view",
    ),
    path(
        "update_users/<int:pk>/",
        permission_required("authorities.change_authority")(
            AuthorityEditUsersView.as_view()
        ),
        name="authority_update_users",
    ),
]



from django.contrib import admin
from .models import Authority, AuthorityKind


class AuthorityKindAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
    )
    search_fields = ("name",)


class AuthorityAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "parent",
    )
    search_fields = ("name",)
    filter_horizontal = ("users",)


admin.site.register(AuthorityKind, AuthorityKindAdmin)
admin.site.register(Authority, AuthorityAdmin)
